/*****************************************************************
 * BotInterface.h
 * 
 * This is the header file for the code necessary to interface with 
 * the FlexBot MOD in Half-Life. If you are writing a bot
 * behavior, there is no need to modify this file. All behavior-
 * specific code should be placed in the "npc.cpp" and "npc.h"
 * files.
 *
 * This implementation of the behavior interface is a simple yet
 * robust way to make sure that your FlexBot behavior can control
 * multiple bots simultanously without stomping on other bots'
 * state information. This is achieved by creating a new npc
 * object for each active bot.
 *****************************************************************/

#ifndef BEHAVIOR_INTERFACE_H
#define BEHAVIOR_INTERFACE_H

#include "grl-agent.h"

#define EXPORT_BOT __declspec (dllexport)

// Half-Life supports a maximum of 32 clients
#define MAX_BOTS 32

/*****************************************************
 * EXPORTED FUNCTIONS								 *
 *****************************************************/
EXPORT_BOT int CreateBotBehavior();
EXPORT_BOT void RemoveBotBehavior(int instance);
EXPORT_BOT void Think(int instance);
EXPORT_BOT int GetRotate(int instance);
EXPORT_BOT int GetTranslate(int instance);
EXPORT_BOT int GetStrafe(int instance);
EXPORT_BOT int GetPitch(int instance);
EXPORT_BOT bool GetShoot(int instance);
EXPORT_BOT bool GetDuck(int instance);
EXPORT_BOT bool GetJump(int instance);
EXPORT_BOT bool GetSecondShoot(int instance);
EXPORT_BOT int GetWeapon(int instance);
EXPORT_BOT bool GetReload(int instance);
EXPORT_BOT bool GetUse(int instance);

/*****************************************************
 * TYPEDEFS FOR FUNCTION POINTERS					 *
 *****************************************************/
typedef int (* PFN_VISION_DIST) (float, int);
typedef int (* PFN_VISION_DIST_WITH_PITCH) (float, float, int);
typedef int (* PFN_VISION_SWEEP_DIST) (float, int, int, int);
typedef int (* PFN_PREDEF_VISION_DIST) (int);
typedef float (* PFN_YAW_TO_PLAYER) (int);
typedef float (* PFN_PITCH_TO_PLAYER) (int);
typedef int (* PFN_DIST_TO_PLAYER) (int);
typedef bool (* PFN_IS_PLAYER_VISIBLE) (int);
typedef int (* PFN_HEALTH) ();
typedef float (* PFN_ROTATION) ();
typedef int (* PFN_PLAYER_INDEX_RETURN) ();
typedef bool (* PFN_IS_MOVING) ();
typedef void (* PFN_DEBUG_SIGNAL) (const char *);
typedef int (* PFN_DIST_TO_WAYPOINT) (int);
typedef float (* PFN_YAW_TO_WAYPOINT) (int);
typedef int (* PFN_NEAREST_WAYPOINT) ();
typedef int (* PFN_DIST_TO_INFOPOINT) (int, int);
typedef float (* PFN_YAW_TO_INFOPOINT) (int, int);
typedef int (* PFN_NEAREST_INFOPOINT) (int);
typedef int (* PFN_NEXT_IP_TO_EXPIRE) (int);
typedef int (* PFN_AMMO_IN_CLIP) ();
typedef int (* PFN_NEAREST_VISIBLE_TEAMMATE)();
typedef float (* PFN_YAW_TO_ENEMY_SOUND)();
typedef int (* PFN_DISTANCE_TO_ENEMY_SOUND)();
typedef int (* PFN_NEAREST_ITEM)(int);
typedef float (* PFN_YAW_TO_ITEM)(int, int);
typedef int (* PFN_DISTANCE_TO_ITEM)(int, int);
typedef float (* PFN_PITCH_TO_ITEM)(int, int);
typedef bool (* PFN_CAN_GET_ITEM)(int, int);
typedef int (* PFN_GET_CURRENT_WEAPON)();
typedef bool (* PFN_IS_FRIEND)(int);
typedef int (* PFN_GET_X)();
typedef int (* PFN_GET_Y)();
typedef int (* PFN_GET_Z)();
typedef float (* PFN_PITCH_TO_WAYPOINT)(int);
typedef float (* PFN_PITCH_TO_INFOPOINT)(int, int);
typedef bool (* PFN_GET_POSSESSES_GUN)(int);
typedef int (* PFN_PRIMARY_AMMO_IN_STOCK)(int);
typedef int (* PFN_SECONDARY_AMMO_IN_STOCK)(int);
typedef bool (* PFN_ON_A_LADDER)();
typedef void (* PFN_ADD_WAYPOINT)();
typedef void (* PFN_ADD_INFOPOINT)(int);

/*****************************************************
 * BEHAVIOR DLL INITIALIZATION						 *
 *****************************************************/
bool InitSensors();


/*****************************************************
 * WRAPPER FUNCTIONS FOR SENSORS					 *
 *****************************************************/

// -- DEBUGGING --
void update_trace(const char *, int);
void update_trace(const char *, float);
void update_trace(const char *, bool);

// -- SENSORS --
int get_vision_distance(float, int);
int get_vision_distance_with_pitch(float, float, int);
int get_vision_sweep_distance(float, int, int, int);
int get_left_distance(int);
int get_right_distance(int);
int get_center_distance(int);
float yaw_to_player(int);
float pitch_to_player(int);
int distance_to_player(int);
bool is_player_visible(int);
int get_health();
float get_absolute_yaw();
float get_absolute_pitch();
int nearest_visible_enemy();
bool is_moving();
float yaw_to_waypoint(int);
int distance_to_waypoint(int);
int nearest_waypoint();
float yaw_to_infopoint(int, int);
int distance_to_infopoint(int, int);
int nearest_infopoint(int);
int next_ip_to_expire(int);
int ammo_in_clip();
int nearest_visible_teammate();
float yaw_to_enemy_sound();
int distance_to_enemy_sound();
int nearest_item(int itemType);
float yaw_to_item(int itemType, int index);
int distance_to_item(int itemType, int index);
float pitch_to_item(int itemType, int index);
bool can_get_item(int itemType, int index);
int get_current_weapon();
int get_x();
int get_y();
int get_z();
bool is_friend(int index);
float pitch_to_infopoint(int pointType, int index);
float pitch_to_waypoint(int index);
int primary_ammo_in_stock(int gunID);
int secondary_ammo_in_stock(int gunID);
bool get_possesses_gun(int gunID);
bool on_a_ladder();
void add_waypoint();
void add_infopoint(int lifespan);


#endif