/*****************************************************************
 * FlexMonitor.h
 * 
 * This is the header file for the code necessary to write a
 * game monitor module for the FlexBot MOD in Half-Life. If you
 * are writing a FlexMonitor module, there is no need to modify this
 * file. All monitor-specific code can be placed in the
 * "FlexMonitor.cpp" file.
 *
 * If placed in the FlexModule directory within the FlexBot directory,
 * this control module will have its PlayerUpdate() function called
 * called every think cycle by each player. Keep in mind that 
 * calls to this control module are synchronous (blocking), so it is
 * important to make your FlexMonitor module code efficient so as not to
 * adversely impact game performance.
 *****************************************************************/

#ifdef FLEXMONITOR_EXPORTS
#define FLEXMONITOR_API __declspec(dllexport)
#else
#define FLEXMONITOR_API __declspec(dllimport)
#endif

// Typedef for function pointers to external FlexBot routines
typedef const char * (* PFN_CURRENT_MAP)();
typedef bool (* PFN_IS_PLAYER_BOT)(int);
typedef const char * (* PFN_BOT_BEHAVIOR_NAME)(int);
typedef const char * (* PFN_PLAYER_TEAM)(int);
typedef const char * (* PFN_GAME_TYPE)();
typedef const char * (* PFN_PLAYER_WEAPON)(int);
typedef const char * (* PFN_PLAYER_NAME)(int);
typedef int (* PFN_PLAYER_KILLS)(int);
typedef int (* PFN_PLAYER_DEATHS)(int);

// Function pointers to Half-Life
HINSTANCE moduleHandle;
PFN_CURRENT_MAP pfnCurrentMap;
PFN_IS_PLAYER_BOT pfnIsPlayerBot;
PFN_BOT_BEHAVIOR_NAME pfnBotBehaviorName;
PFN_PLAYER_TEAM pfnPlayerTeam;
PFN_GAME_TYPE pfnGameType;
PFN_PLAYER_WEAPON pfnPlayerWeapon;
PFN_PLAYER_NAME pfnPlayerName;
PFN_PLAYER_KILLS pfnPlayerKills;
PFN_PLAYER_DEATHS pfnPlayerDeaths;

// Have the function pointers to FlexBot been loaded yet?
bool initialized = FALSE;

// ------ Exported by this FlexMonitor module ------ //
FLEXMONITOR_API void PlayerUpdate(int, int, int, int, int, float, float, const char *);

// ------ Wrapper functions to access FlexBot functions ------ //
const char * CurrentMap();
bool IsPlayerBot(int);
const char * BotBehaviorName(int);
const char * PlayerTeam(int);
const char * GameType();
const char * PlayerWeapon(int);
const char * PlayerName(int);
int PlayerKills(int);
int PlayerDeaths(int);

/*****************************************************
 * CURRENT MAP		         					     *
 *													 *
 * Calling this function will return the map		 *
 * currently being used in the game.				 *
 *****************************************************/
const char * CurrentMap() {
	return (pfnCurrentMap)();
}

/*****************************************************
 * IS PLAYER BOT	         					     *
 *													 *
 * Calling this function will return whether or not  *
 * the player index refers to a bot.				 *
 *****************************************************/
bool IsPlayerBot(int playerNum) {
	return (pfnIsPlayerBot)(playerNum);
}

/*****************************************************
 * BOT BEHAVIOR NAME         					     *
 *													 *
 * Calling this function will return the name of	 *
 * the bot behavior being used by that player. Will	 *
 * return "*none*" if the player is human.			 *
 *****************************************************/
const char * BotBehaviorName(int playerNum) {
	return (pfnBotBehaviorName)(playerNum);
}

/*****************************************************
 * PLAYER TEAM		         					     *
 *													 *
 * Calling this function will return the name of the *
 * player's team. Returns a value of "*none*" if	 *
 * the bot is not assigned to a team.				 *
 *****************************************************/
const char * PlayerTeam(int playerNum) {
	return (pfnPlayerTeam)(playerNum);
}

/*****************************************************
 * GAME TYPE		         					     *
 *													 *
 * Calling this function will return the type of	 *
 * game currently being run (e.g. "deathmatch" or	 *
 * "teamplay".										 *
 *****************************************************/
const char * GameType() {
	return (pfnGameType)();
}

/*****************************************************
 * PLAYER WEAPON	         					     *
 *													 *
 * Returns the weapon currently being used by the	 *
 * specified player.								 *
 *****************************************************/
const char * PlayerWeapon(int playerNum) {
	return (pfnPlayerWeapon)(playerNum);
}

/*****************************************************
 * PLAYER NAME		         					     *
 *													 *
 * Returns the player's in-game name.				 *
 *****************************************************/
const char * PlayerName(int playerNum) {
	return (pfnPlayerName)(playerNum);
}

/*****************************************************
 * PLAYER KILLS		         					     *
 *													 *
 * Returns the player's kill count.					 *
 *****************************************************/
int PlayerKills(int playerNum) {
	return (pfnPlayerKills)(playerNum);
}

/*****************************************************
 * PLAYER KILLS		         					     *
 *													 *
 * Returns the player's death count.				 *
 *****************************************************/
int PlayerDeaths(int playerNum) {
	return (pfnPlayerDeaths)(playerNum);
}